#!/bin/bash
#
# This file is part of rasdaman community.
#
# Rasdaman community is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Rasdaman community is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rasdaman community.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 Peter Baumann /
# rasdaman GmbH.
#
# For more information please see <http://www.rasdaman.org>
# or contact Peter Baumann via <baumann@rasdaman.com>.
#
# ------------------------------------------------------------------------------
#
# SYNOPSIS
# ./petascope_insertdemo.sh
#
# Description
#  Command-line utility for updating the petascope database. The updating is
#  done by importing all appropriate updateX.sql files to postgres. The script
#  determines automatically which updates need to be imported. There are two
#  cases:
#   1. no petascope database is present, in which case the updates start from 0.
#   2. a ps_dbupdates table is present, in this case the updates start from the
#      number saved in the update column.
#
#  Database connection details are read from etc/petascope.properties
#
# PRECONDITIONS
#  1) PostgreSQL server must be running
#  2) etc/petascope.properties should be present, and the metadata_user should
#     have appropriate write rights in postgres.
#  3) share/rasdaman/petascope should contain the SQL update scripts

PROG=`basename $0`

CODE_OK=0
CODE_FAIL=255

# petascope settings file
SETTINGS=/etc/petascope.properties

# petascope updateN.sql scripts
SCRIPT_DIR=/usr/share/rasdaman/petascope

# ------------------------------------------------------------------------------
# functions
# ------------------------------------------------------------------------------

#
# logging
#
log() {
  echo "$PROG: $*"
}
logn() {
  echo -n "$PROG: $*"
}
error() {
  echo "$PROG: $*" >&2
  echo "$PROG: exiting." >&2
  exit $CODE_FAIL
}
feedback() {
  if [ $? -ne 0 ]; then
    echo failed.
    echo "$PROG: exiting." >&2
    exit $CODE_FAIL
  else
    echo ok.
  fi
}

#
# checks
#
check_postgres() {
  which psql > /dev/null || error "PostgreSQL missing, please add psql to the PATH."
  pgrep postgres > /dev/null
  if [ $? -ne 0 ]; then
    pgrep postmaster > /dev/null || error "The PostgreSQL service is not started."
  fi
  psql -U $PS_USER -p $PS_PORT --list 2>&1 | egrep "\b$PS_DB\b" > /dev/null
  if [ $? -ne 0 ]; then
    error "no petascope database found, please run update_petascopedb.sh first."
  fi
}
check_rasdaman() {
  which rasql > /dev/null || error "rasdaman not installed, please add rasql to the PATH."
  pgrep rasmgr > /dev/null || error "rasdaman not started, please start with start_rasdaman.sh"

  # set rascontrol login
  if [ -z "$RASLOGIN" ]; then
    export RASLOGIN=rasadmin:d293a15562d3e70b6fdc5ee452eaed40
  fi
  $RASCONTROL -x 'list srv -all' > /dev/null || error "no rasdaman servers started."
}
check_paths() {
  if [ ! -f "$SETTINGS" ]; then
	  error "petascope settings not found: $SETTINGS"
  fi
  if [ ! -d "$SCRIPT_DIR" ]; then
    error "SQL update script directory not found: $SCRIPT_DIR"
  fi
}

# ------------------------------------------------------------------------------
# work
# ------------------------------------------------------------------------------

check_paths

#
# postgres connection details
#
PS_USER=`grep metadata_user "$SETTINGS" | awk -F "=" '{print $2}'`
PS_USER="${PS_USER#"${PS_USER%%[![:space:]]*}"}"
PS_PASS=`grep metadata_pass "$SETTINGS" | awk -F "=" '{print $2}'`
PS_PASS="${PS_PASS#"${PS_PASS%%[![:space:]]*}"}"
PS_DB=`grep metadata_url "$SETTINGS" | awk -F "/" '{print $4}' | tr -d '\n'`
PS_PORT=`grep metadata_url "$SETTINGS" | awk -F ":|/" '{print $6}' | tr -d '\n'`

# add user/pass to the .pgpass file
PGPASS_LINE="$HOSTNAME:*:*:$PS_USER:$PS_PASS"
PGPASS_FILE="$HOME/.pgpass"
if [ ! -f $PGPASS_FILE ]; then
  echo "$PGPASS_LINE" > $PGPASS_FILE
  chmod 600 $PGPASS_FILE
else
  grep "$PS_USER" $PGPASS_FILE > /dev/null
  if [ $? -ne 0 ]; then
    echo "$PGPASS_LINE" >> $PGPASS_FILE
  fi
fi

#
# postgress commands
#
PSQL="psql -U $PS_USER -d $PS_DB -p $PS_PORT"

# print some info
log "postgres settings read from $SETTINGS"
log "  user: $PS_USER"
log "  port: $PS_PORT"
log "  db: $PS_DB"

check_postgres

#
# rasdaman connection details
#
RAS_USER=`grep rasdaman_admin_user "$SETTINGS" | awk -F "=" '{print $2}' | tr -d '\n'`
RAS_PASS=`grep rasdaman_admin_pass "$SETTINGS" | awk -F "=" '{print $2}' | tr -d '\n'`
RAS_DB=`grep rasdaman_database "$SETTINGS" | awk -F "=" '{print $2}' | tr -d '\n'`
RAS_SERVER=`grep rasdaman_url "$SETTINGS" | awk -F ":|/" '{print $4}' | tr -d '\n'`
RAS_PORT=`grep rasdaman_url "$SETTINGS" | awk -F ":|/" '{print $5}' | tr -d '\n'`

#
# rasdaman commands
#
RASQL="/usr/bin/rasql --user $RAS_USER --passwd $RAS_PASS -d $RAS_DB -s $RAS_SERVER -p $RAS_PORT"
RASCONTROL="/usr/bin/rascontrol"

# print some info
log "rasdaman settings"
log "  user: $RAS_USER"
log "  host: $RAS_SERVER"
log "  port: $RAS_PORT"
log "  db: $RAS_DB"

check_rasdaman

#
# insert sample data to rasdaman
#
c="mean_summer_airtemp"
log "sample coverage $c"

logn "  inserting into rasdaman... "

$RASQL -q 'select r from RAS_COLLECTIONNAMES as r' --out string | grep "$c" > /dev/null
if [ $? -ne 0 ]; then
  $RASQL -q "create collection $c GreySet" > /dev/null 2>&1
  if [ $? -ne 0 ]; then
    echo "failed creating collection."
    $RASQL -q "create collection $c GreySet"
    exit $CODE_FAIL
  else
    $RASQL -q "insert into $c values inv_tiff(\$1)" --file $SCRIPT_DIR/mean_summer_airtemp.tif > /dev/null 2>&1
    feedback
  fi
else
  echo "already inserted."
fi

#
# insert metadata into petascope
#
logn "  inserting into petascope... "
id=`$PSQL -c "select id from PS_Coverage where name = '$c' " | head -3 | tail -1`
if [ "$id" == "(0 rows)" ]; then
  $PSQL -c "insert into PS_Coverage (name, nulldefault, interpolationtypedefault, nullresistancedefault, type) values ( '$c','0', 5, 2, 'RectifiedGridCoverage')" > /dev/null
  c_id=$($PSQL -c  "select id from PS_Coverage where name = '$c' " | head -3 | tail -1) > /dev/null
  $PSQL -c "insert into PS_CellDomain (coverage, i, lo, hi )  values ( $c_id, 0, 0, 885)" > /dev/null
  $PSQL -c "insert into PS_CellDomain (coverage, i, lo, hi )  values ( $c_id, 1, 0, 710)" > /dev/null
  $PSQL -c "insert into PS_Domain (coverage, i, name, type, numLo, numHi) values ( $c_id, 0, 'x', 1, 111.975, 156.275 )" > /dev/null
  $PSQL -c "insert into PS_Domain (coverage, i, name, type, numLo, numHi) values ( $c_id, 1, 'y', 2, -44.525, -8.975 )" > /dev/null
  $PSQL -c "insert into PS_Range (coverage, i, name, type) values ($c_id, 0, 'pan', 7)" > /dev/null
  $PSQL -c "insert into PS_InterpolationSet (coverage, interpolationType, nullResistance) values ( $c_id, 5, 2)" > /dev/null
  $PSQL -c "insert into PS_NullSet (coverage, nullValue) values ( $c_id, '0')" > /dev/null
  $PSQL -c "insert into PS_CrsDetails (coverage, low1, high1, low2, high2) values ( $c_id, 111.975, 156.275, -44.525, -8.975)" > /dev/null
  x_id=$($PSQL -c "select id from PS_domain where coverage = $c_id and type=1" | head -3 | tail -1) > /dev/null
  y_id=$($PSQL -c "select id from PS_domain where coverage = $c_id and type=2" | head -3 | tail -1) > /dev/null
  $PSQL -c "insert into PS_Crsset ( axis, crs) values ( $x_id, 9)" > /dev/null
  $PSQL -c "insert into PS_Crsset ( axis, crs) values ( $y_id, 9)" > /dev/null
  $PSQL -c "insert into PS_Descriptions (coverage, title, abstract, keywords) values ( $c_id, 'mean_summer_airtemp', 'This coverage shows the mean summer air temperatures over Australia in the period 2001-2007', 'summer temperature, australia')" > /dev/null
  feedback
else
  echo "already inserted."
fi
